package xyz.lychee.lagfixer.libs.adventure.text.event;

import java.time.Duration;
import java.time.temporal.TemporalAmount;
import java.util.function.Consumer;
import java.util.function.Predicate;
import xyz.lychee.lagfixer.libs.adventure.audience.Audience;
import xyz.lychee.lagfixer.libs.adventure.builder.AbstractBuilder;
import xyz.lychee.lagfixer.libs.adventure.permission.PermissionChecker;
import xyz.lychee.lagfixer.libs.adventure.util.PlatformAPI;
import xyz.lychee.lagfixer.libs.annotations.CheckReturnValue;
import xyz.lychee.lagfixer.libs.annotations.Contract;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.annotations.ApiStatus.Internal;
import xyz.lychee.lagfixer.libs.annotations.ApiStatus.NonExtendable;
import xyz.lychee.lagfixer.libs.examination.Examinable;

@FunctionalInterface
public interface ClickCallback<T extends Audience> {
   Duration DEFAULT_LIFETIME = Duration.ofHours(12L);
   int UNLIMITED_USES = -1;

   @CheckReturnValue
   @Contract(
      pure = true
   )
   static <W extends Audience, N extends W> @NotNull ClickCallback<W> widen(final @NotNull ClickCallback<N> original, final @NotNull Class<N> type, final @Nullable Consumer<? super Audience> otherwise) {
      return (audience) -> {
         if (type.isInstance(audience)) {
            original.accept((Audience)type.cast(audience));
         } else if (otherwise != null) {
            otherwise.accept(audience);
         }

      };
   }

   @CheckReturnValue
   @Contract(
      pure = true
   )
   static <W extends Audience, N extends W> @NotNull ClickCallback<W> widen(final @NotNull ClickCallback<N> original, final @NotNull Class<N> type) {
      return widen(original, type, (Consumer)null);
   }

   void accept(final @NotNull T audience);

   @CheckReturnValue
   @Contract(
      pure = true
   )
   default @NotNull ClickCallback<T> filter(final @NotNull Predicate<T> filter) {
      return this.filter(filter, (Consumer)null);
   }

   @CheckReturnValue
   @Contract(
      pure = true
   )
   default @NotNull ClickCallback<T> filter(final @NotNull Predicate<T> filter, final @Nullable Consumer<? super Audience> otherwise) {
      return (audience) -> {
         if (filter.test(audience)) {
            this.accept(audience);
         } else if (otherwise != null) {
            otherwise.accept(audience);
         }

      };
   }

   @CheckReturnValue
   @Contract(
      pure = true
   )
   default @NotNull ClickCallback<T> requiringPermission(final @NotNull String permission) {
      return this.requiringPermission(permission, (Consumer)null);
   }

   @CheckReturnValue
   @Contract(
      pure = true
   )
   default @NotNull ClickCallback<T> requiringPermission(final @NotNull String permission, final @Nullable Consumer<? super Audience> otherwise) {
      return this.filter((audience) -> {
         return ((PermissionChecker)audience.getOrDefault(PermissionChecker.POINTER, ClickCallbackInternals.ALWAYS_FALSE)).test(permission);
      }, otherwise);
   }

   @PlatformAPI
   @Internal
   public interface Provider {
      @NotNull ClickEvent create(final @NotNull ClickCallback<Audience> callback, @NotNull final @NotNull Options options);
   }

   @NonExtendable
   public interface Options extends Examinable {
      @NotNull
      static @NotNull Builder builder() {
         return new ClickCallbackOptionsImpl.BuilderImpl();
      }

      @NotNull
      static @NotNull Builder builder(@NotNull final @NotNull Options existing) {
         return new ClickCallbackOptionsImpl.BuilderImpl(existing);
      }

      int uses();

      @NotNull Duration lifetime();

      @NonExtendable
      public interface Builder extends AbstractBuilder<Options> {
         @NotNull
         @NotNull Builder uses(int useCount);

         @NotNull
         @NotNull Builder lifetime(final @NotNull TemporalAmount duration);
      }
   }
}
